-------------------------------------------------------------------------------
-- PublicDungeonChampions v1.2.3
-------------------------------------------------------------------------------
--
-- Update and Changing for Public Dungeon Champions by MerlinGer 2014
-- The Changes are in Copyright by MerlinGer
--
-- Copyright (c) 2014, 2015 Ales Machat (Garkin)
--
-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:
--
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.
--
-------------------------------------------------------------------------------

--Libraries--------------------------------------------------------------------
local LAM = LibStub("LibAddonMenu-2.0")
local LMP = LibStub("LibMapPins-1.0")

--Local constants -------------------------------------------------------------
local ADDON_VERSION = "1.2.3"
local PINS_UNKNOWN = "PDCSMapPin_unknown"
local PINS_COLLECTED = "PDCSMapPin_collected"
local PINS_COMPASS_UNKNOWN = "PDCSCompassPin_unknown"
local PINS_COMPASS_KNOWN = "PDCSCompassPin_known"

--Local variables -------------------------------------------------------------
local updatePins = {}
local updating = false
local savedVariables
local defaults = {         -- default settings for saved variables
   compassMaxDistance = 0.05,
   pinTexture = {
      type = 1,
      size = 38,
      level = 40,
   },
   completeColor = { 0, 1, 0, .70 },
   incompleteColor = { 1, 0, 0, .70 }, 
   filters = {
      [PINS_COMPASS_UNKNOWN] = true,
      [PINS_COMPASS_KNOWN] = false,
      [PINS_UNKNOWN] = true,
      [PINS_COLLECTED] = true,
   },
}

-- Local functions ------------------------------------------------------------
local function MyPrint(...)
   CHAT_SYSTEM:AddMessage(...)
end

-- Pins -----------------------------------------------------------------------
local pinTexturesList = {
   [1] = "Default icons",
   [2] = "Normal Ingame Icons",
}

local pinTextures = {
   unknown = {
      -- [1] = "PublicDungeonChampions/Icons/GroupBoss.dds",
      [1] = "/esoui/art/icons/poi/poi_groupboss_complete.dds",
      [2] = "/esoui/art/icons/poi/poi_groupboss_incomplete.dds",
   },
   collected = {
      -- [1] = "PublicDungeonChampions/Icons/GroupBoss.dds",
      [1] = "/esoui/art/icons/poi/poi_groupboss_complete.dds",
      [2] = "/esoui/art/icons/poi/poi_groupboss_complete.dds",
   },
}

--tooltip creator
local pinTooltipCreator = {}
pinTooltipCreator.tooltip = 1
pinTooltipCreator.creator = function(pin)
   local _, pinTag = pin:GetPinTypeAndTag()
   local name = GetAchievementInfo(pinTag[3])
   local description, numCompleted = GetAchievementCriterion(pinTag[3], pinTag[4])
   local info = {}

   if pinTag[5] ~= nil then
      table.insert(info, "[" .. GetString("PDCS_MOREINFO", pinTag[5]) .. "]")
   end
   if numCompleted == 1 then
      table.insert(info, "[" .. GetString(PDCS_KNOWN) .. "]")
   end

   InformationTooltip:AddLine(zo_strformat("<<1>>", name), "ZoFontGameOutline", ZO_SELECTED_TEXT:UnpackRGB())
   ZO_Tooltip_AddDivider(InformationTooltip)
   InformationTooltip:AddLine(zo_strformat("(<<1>>) <<2>>", pinTag[4], description), "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
   if info[1] then
      InformationTooltip:AddLine(table.concat(info, " / "), "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
   end
end

--add callbacks
local function CreatePins()
   local zone, subzone = LMP:GetZoneAndSubzone()
   local PublicDungeonChampions = PublicDungeonChampions_GetLocalData(zone, subzone)

   if PublicDungeonChampions ~= nil then
      for _, pinData in ipairs(PublicDungeonChampions) do
         local _, numCompleted = GetAchievementCriterion(pinData[3], pinData[4])
         if numCompleted == 1 then
            if updatePins[PINS_COLLECTED] and LMP:IsEnabled(PINS_COLLECTED) then
               LMP:CreatePin(PINS_COLLECTED, pinData, pinData[1], pinData[2])
            end
            if updatePins[PINS_COMPASS_KNOWN] and savedVariables.filters[PINS_COMPASS_KNOWN] then
               COMPASS_PINS.pinManager:CreatePin(PINS_COMPASS_KNOWN, pinData, pinData[1], pinData[2])
            end
         elseif numCompleted == 0 then
            if updatePins[PINS_UNKNOWN] and LMP:IsEnabled(PINS_UNKNOWN) then
               LMP:CreatePin(PINS_UNKNOWN, pinData, pinData[1], pinData[2])
            end
            if updatePins[PINS_COMPASS_UNKNOWN] and savedVariables.filters[PINS_COMPASS_UNKNOWN] then
               COMPASS_PINS.pinManager:CreatePin(PINS_COMPASS_UNKNOWN, pinData, pinData[1], pinData[2])
            end
         end
      end
   end

   updatePins = {}
   updating = false
end

local function QueueCreatePins(pinType)
   updatePins[pinType] = true

   if not updating then
      updating = true
      if IsPlayerActivated() then
         zo_callLater(CreatePins, 50)
      else
         EVENT_MANAGER:RegisterForEvent("PublicDungeonChampions_PinUpdate", EVENT_PLAYER_ACTIVATED,
            function(event)
               EVENT_MANAGER:UnregisterForEvent("PublicDungeonChampions_PinUpdate", event)
               CreatePins()
            end)
      end
   end
end

local function MapCallback_unknown()
   if not LMP:IsEnabled(PINS_UNKNOWN) or (GetMapContentType() ~= MAP_CONTENT_DUNGEON) then return end
   QueueCreatePins(PINS_UNKNOWN)
end

local function MapCallback_collected()
   if not LMP:IsEnabled(PINS_COLLECTED) or (GetMapContentType() ~= MAP_CONTENT_DUNGEON) then return end
   QueueCreatePins(PINS_COLLECTED)
end

local function CompassCallback_unknown()
   if not savedVariables.filters[PINS_COMPASS_UNKNOWN] or (GetMapContentType() ~= MAP_CONTENT_DUNGEON) then return end
   QueueCreatePins(PINS_COMPASS_UNKNOWN)
end

local function CompassCallback_known()
   if not savedVariables.filters[PINS_COMPASS_KNOWN] or (GetMapContentType() ~= MAP_CONTENT_DUNGEON) then return end
   QueueCreatePins(PINS_COMPASS_KNOWN)
end


-- Slash commands -------------------------------------------------------------
local function ShowMyPosition()
   local x, y = GetMapPlayerPosition("player")

   local locX = ("%05.02f"):format(zo_round(x*10000)/100)
   local locY = ("%05.02f"):format(zo_round(y*10000)/100)

   MyPrint(zo_strformat("<<1>>: <<2>>\195\151<<3>> (<<4>>)", GetMapName(), locX, locY, LMP:GetZoneAndSubzone(true)))
   
end
SLASH_COMMANDS["/pdcmypos"] = ShowMyPosition
SLASH_COMMANDS["/pdcmyposition"] = ShowMyPosition
SLASH_COMMANDS["/pdcmyloc"] = ShowMyPosition
SLASH_COMMANDS["/pdcmylocation"] = ShowMyPosition

-- Settings menu --------------------------------------------------------------
local function CreateSettingsMenu()
   local panelData = {
      type = "panel",
      name = GetString(PDCS_TITLE),
      displayName = "|cFFFFB0" .. GetString(PDCS_TITLE) .. "|r",
      author = "MerlinGer (featured by Garkin)",
      version = ADDON_VERSION,
      slashCommand = "/pdc",
      registerForRefresh = true,
      registerForDefaults = true,
   }
   LAM:RegisterAddonPanel("PublicDungeonChampions", panelData)

   local CreateIcons, unknownIcon, collectedIcon
   CreateIcons = function(panel)
      if panel == PublicDungeonChampions then
         unknownIcon = WINDOW_MANAGER:CreateControl(nil, panel.controlsToRefresh[1], CT_TEXTURE)
         unknownIcon:SetAnchor(RIGHT, panel.controlsToRefresh[2].dropdown:GetControl(), LEFT, -10, 0)
         unknownIcon:SetTexture(pinTextures.unknown[savedVariables.pinTexture.type])
         unknownIcon:SetDimensions(savedVariables.pinTexture.size, savedVariables.pinTexture.size)
         collectedIcon = WINDOW_MANAGER:CreateControl(nil, panel.controlsToRefresh[2], CT_TEXTURE)
         collectedIcon:SetAnchor(RIGHT, unknownIcon, LEFT, -5, 0)
         collectedIcon:SetTexture(pinTextures.collected[savedVariables.pinTexture.type])
         collectedIcon:SetDimensions(savedVariables.pinTexture.size, savedVariables.pinTexture.size)
         if savedVariables.pinTexture.type == 1 then
            unknownIcon:SetColor(unpack(savedVariables.incompleteColor))
            collectedIcon:SetColor(unpack(savedVariables.completeColor))
         end
         CALLBACK_MANAGER:UnregisterCallback("LAM-PanelControlsCreated", CreateIcons)
      end
   end
   CALLBACK_MANAGER:RegisterCallback("LAM-PanelControlsCreated", CreateIcons)

   local optionsTable = {
      {
         type = "header",
         name = GetString(PDCS_MAPPINS_HEADER),
         width = "full",
      },
      {
         type = "dropdown",
         name = GetString(PDCS_PIN_TEXTURE),
         tooltip = GetString(PDCS_PIN_TEXTURE_DESC),
         choices = pinTexturesList,
         getFunc = function() return pinTexturesList[savedVariables.pinTexture.type] end,
         setFunc = function(selected)
               for index, name in ipairs(pinTexturesList) do
                  if name == selected then
                     savedVariables.pinTexture.type = index
                     LMP:SetLayoutKey(PINS_UNKNOWN, "texture", pinTextures.unknown[index])
                     LMP:SetLayoutKey(PINS_COLLECTED, "texture", pinTextures.collected[index])
                     COMPASS_PINS.pinLayouts[PINS_COMPASS_UNKNOWN].texture = pinTextures.unknown[index]
                     COMPASS_PINS.pinLayouts[PINS_COMPASS_KNOWN].texture = pinTextures.collected[index]
                     unknownIcon:SetTexture(pinTextures.unknown[index])
                     collectedIcon:SetTexture(pinTextures.collected[index])
                     if index == 1 then
                        LMP:SetLayoutKey(PINS_UNKNOWN, "tint", ZO_ColorDef:New(unpack(savedVariables.incompleteColor)))
                        LMP:SetLayoutKey(PINS_COLLECTED, "tint", ZO_ColorDef:New(unpack(savedVariables.completeColor)))
                        unknownIcon:SetColor(unpack(savedVariables.incompleteColor))
                        collectedIcon:SetColor(unpack(savedVariables.completeColor))
                     else
                        LMP:SetLayoutKey(PINS_UNKNOWN, "tint", nil)
                        LMP:SetLayoutKey(PINS_COLLECTED, "tint", nil)
                        unknownIcon:SetColor(1,1,1,1)
                        collectedIcon:SetColor(1,1,1,1)
                     end
                     LMP:RefreshPins(PINS_UNKNOWN)
                     LMP:RefreshPins(PINS_COLLECTED)
                     COMPASS_PINS:RefreshPins(PINS_COMPASS_UNKNOWN)
                     COMPASS_PINS:RefreshPins(PINS_COMPASS_KNOWN)
                     break
                  end
               end
            end,
         disabled = function() return not (savedVariables.filters[PINS_UNKNOWN] or savedVariables.filters[PINS_COLLECTED]) end,
         default = pinTexturesList[defaults.pinTexture.type],
      },
      --new items in the optionsTable
      {
         type = "colorpicker",
         name = GetString(PDCS_KNOWN_COL),
         tooltip = GetString(PDCS_KNOWN_COL_TIP),
         getFunc = function() return unpack(savedVariables.completeColor) end,
         setFunc = function( r, g, b, a )
               savedVariables.completeColor = { r, g, b, a }
               LMP:SetLayoutKey(PINS_COLLECTED, "tint", ZO_ColorDef:New( r, g, b, a ))
               collectedIcon:SetColor(r, g, b, a)
         end,
         disabled = function() return not (savedVariables.filters[PINS_COLLECTED] and savedVariables.pinTexture.type == 1) end,
         default = { r = defaults.completeColor[1], g = defaults.completeColor[2], b = defaults.completeColor[3], a = defaults.completeColor[4] }
      },
      {
         type = "colorpicker",
         name = GetString(PDCS_UNKNOWN_COL),
         tooltip = GetString(PDCS_UNKNOWN_COL_TIP),
         getFunc = function() return unpack(savedVariables.incompleteColor) end,
         setFunc = function(r, g, b, a)
               savedVariables.incompleteColor = { r, g, b, a }
               LMP:SetLayoutKey(PINS_UNKNOWN, "tint", ZO_ColorDef:New( r, g, b, a ))
               unknownIcon:SetColor(r, g, b, a)
         end,
         disabled = function() return not (savedVariables.filters[PINS_UNKNOWN] and savedVariables.pinTexture.type == 1) end,
         default = { r = defaults.incompleteColor[1], g = defaults.incompleteColor[2], b = defaults.incompleteColor[3], a = defaults.incompleteColor[4] }
      },
      {
         type = "slider",
         name = GetString(PDCS_PIN_SIZE),
         tooltip = GetString(PDCS_PIN_SIZE_DESC),
         min = 20,
         max = 70,
         getFunc = function() return savedVariables.pinTexture.size end,
         setFunc = function(size)
               savedVariables.pinTexture.size = size
               unknownIcon:SetDimensions(size, size)
               collectedIcon:SetDimensions(size, size)
               LMP:SetLayoutKey(PINS_UNKNOWN, "size", size)
               LMP:SetLayoutKey(PINS_COLLECTED, "size", size)
               LMP:RefreshPins(PINS_UNKNOWN)
               LMP:RefreshPins(PINS_COLLECTED)
            end,
         disabled = function() return not (savedVariables.filters[PINS_UNKNOWN] or savedVariables.filters[PINS_COLLECTED]) end,
         default = defaults.pinTexture.size
      },
      {
         type = "slider",
         name = GetString(PDCS_PIN_LAYER),
         tooltip = GetString(PDCS_PIN_LAYER_DESC),
         min = 10,
         max = 200,
         step = 5,
         getFunc = function() return savedVariables.pinTexture.level end,
         setFunc = function(level)
               savedVariables.pinTexture.level = level
               LMP:SetLayoutKey(PINS_UNKNOWN, "level", level)
               LMP:SetLayoutKey(PINS_COLLECTED, "level", level)
               LMP:RefreshPins(PINS_UNKNOWN)
               LMP:RefreshPins(PINS_COLLECTED)
            end,
         disabled = function() return not (savedVariables.filters[PINS_UNKNOWN] or savedVariables.filters[PINS_COLLECTED]) end,
         default = defaults.pinTexture.level,
      },
      {
         type = "checkbox",
         name = GetString(PDCS_UNKNOWN),
         tooltip = GetString(PDCS_UNKNOWN_DESC),
         getFunc = function() return savedVariables.filters[PINS_UNKNOWN] end,
         setFunc = function(state)
               savedVariables.filters[PINS_UNKNOWN] = state
               LMP:SetEnabled(PINS_UNKNOWN, state)
            end,
         default = defaults.filters[PINS_UNKNOWN],
      },
      {
         type = "checkbox",
         name = GetString(PDCS_COLLECTED),
         tooltip = GetString(PDCS_COLLECTED_DESC),
         getFunc = function() return savedVariables.filters[PINS_COLLECTED] end,
         setFunc = function(state)
               savedVariables.filters[PINS_COLLECTED] = state
               LMP:SetEnabled(PINS_COLLECTED, state)
            end,
         default = defaults.filters[PINS_COLLECTED]
      },
      {
         type = "header",
         name = GetString(PDCS_COMPASS_HEADER),
         width = "full",
      },
      {
         type = "checkbox",
         name = GetString(PDCS_COMPASS_KNOWN),
         tooltip = GetString(PDCS_COMPASS_KNOWN_DESC),
         getFunc = function() return savedVariables.filters[PINS_COMPASS_KNOWN] end,
         setFunc = function(state)
               savedVariables.filters[PINS_COMPASS_KNOWN] = state
               COMPASS_PINS:RefreshPins(PINS_COMPASS_KNOWN)
            end,
         default = defaults.filters[PINS_COMPASS_KNOWN],
      },
      {
         type = "checkbox",
         name = GetString(PDCS_COMPASS_UNKNOWN),
         tooltip = GetString(PDCS_COMPASS_UNKNOWN_DESC),
         getFunc = function() return savedVariables.filters[PINS_COMPASS_UNKNOWN] end,
         setFunc = function(state)
               savedVariables.filters[PINS_COMPASS_UNKNOWN] = state
               COMPASS_PINS:RefreshPins(PINS_COMPASS_UNKNOWN)
            end,
         default = defaults.filters[PINS_COMPASS_UNKNOWN],
      },
      {
         type = "slider",
         name = GetString(PDCS_COMPASS_DIST),
         tooltip = GetString(PDCS_COMPASS_DIST_DESC),
         min = 1,
         max = 100,
         getFunc = function() return savedVariables.compassMaxDistance * 1000 end,
         setFunc = function(maxDistance)
               savedVariables.compassMaxDistance = maxDistance / 1000
               COMPASS_PINS.pinLayouts[PINS_COMPASS_KNOWN].maxDistance = maxDistance / 1000
               COMPASS_PINS:RefreshPins(PINS_COMPASS_KNOWN)
               COMPASS_PINS.pinLayouts[PINS_COMPASS_UNKNOWN].maxDistance = maxDistance / 1000
               COMPASS_PINS:RefreshPins(PINS_COMPASS_UNKNOWN)
            end,
         disabled = function() return not (savedVariables.filters[PINS_COMPASS_KNOWN] or savedVariables.filters[PINS_COMPASS_UNKNOWN]) end,
         default = defaults.compassMaxDistance * 1000,
      },
   }
   LAM:RegisterOptionControls("PublicDungeonChampions", optionsTable)
end

-- Event handlers -------------------------------------------------------------
local function OnAchievementUpdate(eventCode, achievementId)
   local ids = PublicDungeonChampions_GetAchievementIDs()

   if ids[achievementId] then
      LMP:RefreshPins(PINS_UNKNOWN)
      LMP:RefreshPins(PINS_COLLECTED)
      COMPASS_PINS:RefreshPins(PINS_COMPASS_KNOWN)
      COMPASS_PINS:RefreshPins(PINS_COMPASS_UNKNOWN)
   end
end

local function OnLoad(eventCode, name)
   if name ~= "PublicDungeonChampions" then return end

   savedVariables = ZO_SavedVars:New("PDCS_SavedVariables", 4, nil, defaults)

   --get pin layout from saved variables
   local pinTextureType = savedVariables.pinTexture.type
   local pinTextureLevel = savedVariables.pinTexture.level
   local pinTextureSize = savedVariables.pinTexture.size
   local compassMaxDistance = savedVariables.compassMaxDistance
   local completeColor = savedVariables.completeColor
   local incompleteColor = savedVariables.incompleteColor

   local pinLayout_unknown = { 
      level = pinTextureLevel, 
      texture = pinTextures.unknown[pinTextureType], 
      size = pinTextureSize,
      tint = pinTextureType == 1 and ZO_ColorDef:New(unpack(incompleteColor)) or nil,
   }
   
   local pinLayout_collected = {
      level = pinTextureLevel, 
      texture = pinTextures.collected[pinTextureType], 
      size = pinTextureSize, 
      tint = pinTextureType == 1 and ZO_ColorDef:New(unpack(completeColor)) or nil,
   }
   
   local pinLayout_compassknown = {
      maxDistance = compassMaxDistance,
      texture = pinTextures.unknown[pinTextureType],
      sizeCallback = function(pin, angle, normalizedAngle, normalizedDistance)
         if zo_abs(normalizedAngle) > 0.25 then
            pin:SetDimensions(54 - 24 * zo_abs(normalizedAngle), 54 - 24 * zo_abs(normalizedAngle))
         else
            pin:SetDimensions(48, 48)
         end
      end,
      additionalLayout = {
         function(pin) --set function, color is by default 1,1,1,1, so change it only if texture type is 1
            if savedVariables.pinTexture.type == 1 then
               pin:GetNamedChild("Background"):SetColor(unpack(savedVariables.completeColor))
            end
         end,
         function(pin) --reset function, change color back to default
            pin:GetNamedChild("Background"):SetColor(1,1,1,1)
         end,
      },
   }
   
   local pinLayout_compassunknown = {
      maxDistance = compassMaxDistance,
      texture = pinTextures.unknown[pinTextureType],
      sizeCallback = function(pin, angle, normalizedAngle, normalizedDistance)
         if zo_abs(normalizedAngle) > 0.25 then
            pin:SetDimensions(54 - 24 * zo_abs(normalizedAngle), 54 - 24 * zo_abs(normalizedAngle))
         else
            pin:SetDimensions(48, 48)
         end
      end,
      additionalLayout = {
         function(pin)
            if savedVariables.pinTexture.type == 1 then
               pin:GetNamedChild("Background"):SetColor(unpack(savedVariables.incompleteColor))
            end
         end,
         function(pin)
            pin:GetNamedChild("Background"):SetColor(1,1,1,1)
         end,
      },
   }

   --initialize map pins
   LMP:AddPinType(PINS_UNKNOWN, MapCallback_unknown, nil, pinLayout_unknown, pinTooltipCreator)
   LMP:AddPinType(PINS_COLLECTED, MapCallback_collected, nil, pinLayout_collected, pinTooltipCreator)

   --add filter check boxex
   LMP:AddPinFilter(PINS_UNKNOWN, GetString(PDCS_FILTER_UNKNOWN), nil, savedVariables.filters)
   LMP:AddPinFilter(PINS_COLLECTED, GetString(PDCS_FILTER_COLLECTED), nil, savedVariables.filters)

   --initialize compass pins
   COMPASS_PINS:AddCustomPin(PINS_COMPASS_KNOWN, CompassCallback_known, pinLayout_compassknown)
   COMPASS_PINS:AddCustomPin(PINS_COMPASS_UNKNOWN, CompassCallback_unknown, pinLayout_compassunknown)
   COMPASS_PINS:RefreshPins(PINS_COMPASS_KNOWN)
   COMPASS_PINS:RefreshPins(PINS_COMPASS_UNKNOWN)

   -- addon menu
   CreateSettingsMenu()

   --events
   EVENT_MANAGER:RegisterForEvent("PublicDungeonChampions", EVENT_ACHIEVEMENT_UPDATED, OnAchievementUpdate)

   EVENT_MANAGER:UnregisterForEvent("PublicDungeonChampions", EVENT_ADD_ON_LOADED)
end

EVENT_MANAGER:RegisterForEvent("PublicDungeonChampions", EVENT_ADD_ON_LOADED, OnLoad)
